# <path to game directory>/addons/eventscripts/_engines/python/Lib/esutils/
# strings.py
# by Adam Cunnington

"""Provide custom methods of creating, writing and reading a .ini file for 
string translations.

"""

from __future__ import with_statement
import os
import string

from esutils import tools
import es

__all__ = (
    "DEFAULT_LANGUAGE", 
    "StringsError", 
    "Strings", 
    "Section", 
)

_COMMENT_CHARACTERS = (
    ";", 
    "#",
)

_DELIMIT_CHARACTERS = (
    "=",
    ":",
)

_LANGUAGES = (
    "danish", 
    "dutch", 
    "english", 
    "finish", 
    "french", 
    "german", 
    "italian", 
    "japanese", 
    "korean", 
    "koreana", 
    "norwegian", 
    "portuguese", 
    "russian", 
    "schinese", 
    "spanish", 
    "swedish", 
    "tchinese", 
    "thai", 
)

DEFAULT_LANGUAGE = "english"

class StringsError(Exception):
    """General error encountered relating to esutils.strings"""

class Strings(object):
    """Create an ini file made up of sections and translations that can be 
    written to and read."""

    def __init__(self, file_path):
        """Instantiate a Strings object.
        
        Arguments:
        file_path - the absolute file path to the .ini file.
        
        """
        self._file_path = file_path
        self._sections = set()
        self.tokens = {}
        
    def append(self, section):
        """Add a section to the end.
        
        Arguments:
        section - the section to add.
        
        """
        self._sections.add(section)
        
    def get_string(self, section, language=None, **kw):
        """Get the string for a given section in a specific language (if no 
        language is given, use DEFAULT_LANGUAGE), and return the string, 
        substituting any variables for values. 
        
        Arguments:
        section - the section to get.
        language (Keyword Default: None) - the full name of the language the 
        section should be returned in.
        kw - the keyword variables to substitute.
        
        """
        if language is None or language not in self.tokens[section.name]:
            language = DEFAULT_LANGUAGE
        if not kw:
            return self.tokens[section.name][language]
        template = string.Template(self.tokens[section.name][language])
        return template.substitute(**kw)
        
    def read(self):
        """Read the ini file into a dictionary and raise errors if invalid 
        syntax is encountered.
        
        """
        with open(self._file_path) as fp:
            lines = [line.strip("\n ") for line in fp 
                     if line != "\n" and line[0] not in _COMMENT_CHARACTERS]
        current_section = None
        for line in lines:
            if line.startswith("["):
                if not line.endswith("]"):
                    raise StringsError("invalid syntax for section, %s" 
                                       % line[1: ])
                current_section = line[1: -1]
                self.tokens[current_section] = {}
            else:
                if current_section is None:
                    raise StringsError("language defined before section")
                for delimiter in _DELIMIT_CHARACTERS:
                    if delimiter in line:
                        language, value = (line.strip("\" ") for line 
                                           in line.split(delimiter))
                        self.tokens[current_section][language] = value
                        break
                else:
                    raise StringsError("invalid syntax for language, %s "
                                       "in section, %s" 
                                       %(language, current_section))
                                       
    def write(self, overwrite=False):
        """Write the comments and sections to the ini file initially and 
        thereafter, providing overwrite=True.
        
        Arguments:
        overwrite (Keyword Default: False) - whether or not the file should be 
        overwritten if it already exists.
        
        """
        if os.path.isfile(self._file_path) and not overwrite:
            return
        directory_path = os.path.split(self._file_path)[0]
        if not os.path.isdir(directory_path):
            os.makedirs(directory_path)
        with open(self._file_path, "w") as fp:
            replace_value = "<path to game directory>"
            fp.write("# %s\n\n" 
                              % self._file_path.replace(str(tools.GAME_DIR), 
                                                        replace_value))
            fp.write("# Language Options:")
            for language in _LANGUAGES:
                fp.write("\n# - %s" % language)
            for section in self._sections:
                fp.write("\n\n[%s]" % section.name)
                if not section.translations:
                    continue
                for translation in section.translations:
                    fp.write("\n%s=\"%s\"" 
                                      %(translation, 
                                        section.translations[translation]))
                                        
class Section(object):
    """Create a section that can appear in multiple ini files and can be 
    individually referenced."""
    
    def __init__(self, name):
        """Instantiate a Section object.
        
        Arguments:
        name - the name to reference it by.
        
        """
        self.name = name
        self.translations = {}
        
    def add_translation(self, language, translation):
        """Add a translation to the section.
        
        Arguments:
        language - the full name of the language to add a translation for.
        translation - the string translation.
        
        """
        self.translations[language] = translation